<?php
/**
 * For Antiplagiat cheating.
 */
class AntiplagiatCheat {
    private $dom;
    private $xpath;
    private $rsids;

    public function __construct($filename) {
        $this->dom = new DOMDocument();
        $this->dom->load($filename);
        $this->xpath = new DOMXPath($this->dom);
        $this->xpath->registerNamespace('w', 'http://schemas.openxmlformats.org/wordprocessingml/2006/main');
        $this->rsids = $this->getRsids();
    }

    public function run() {
        $paragraphs = $this->xpath->query('//w:document/w:body/w:p');
        foreach ($paragraphs as $p) {
            $runs = $this->xpath->query('./w:r', $p);
            if (count($runs) == 0) {
                continue; // empty paragraph
            }
            $rsid = $this->xpath->query('./@w:rsidR', $p)->item(0)->value; // current paragraph rsid
            if (!$rsid) { // TODO move rsid definition to breakWr function
                $rsid = $this->rsids[0];
            }
            $rsids = $this->defineTwoRsids($rsid);
            $wordsCount = 0;
            foreach ($runs as $run) {
                $texts = $this->xpath->query('.//w:t', $run);
                if (count($texts) == 0) {
                    continue;
                }
                $rStyle = $this->xpath->query('.//w:rPr', $run)->item(0);
                foreach ($texts as $textNode) {
                    $text = trim($textNode->nodeValue);
                    $words = preg_split('/\\s+/', $text);
                    $wordsCount += count($words);
                    foreach ($words as $word) {
                        $parts = $this->explodeWord($word);
                        // split this $run into separate runs
                        $this->breakWr($parts, $textNode, $run, $p, $rsids);
                    }
                }
                $p->removeChild($run);
            }
        }
    }

    /**
     * Prepends pieces before given run of text ($run)
     * @param $pieces array of strings
     * @param $text DOMNode element
     * @param $run DOMNode element
     * @param $p DOMNode element
     * @param $rStyle DOMNode element
     */
    private function breakWr($pieces, $text, $run, $p, $rsids) {
        $rStyle = $this->xpath->query('.//w:rPr', $run)->item(0);
        for ($i = 0, $n = count($pieces); $i < $n; $i++) {
            $cRun = $run->cloneNode();
            $p->insertBefore($cRun, $run);
            $cText = $text->cloneNode();
            $cText->nodeValue = $pieces[$i];
            if ($i == $n - 1) {
                $cText->nodeValue .= ' ';
                $cText->setAttribute('xml:space', 'preserve');
            }
            if ($rStyle) {
                $cRun->appendChild($rStyle);
            }
            $cRun->appendChild($cText);
            $cRun->setAttribute('w:rsidR', $rsids[$i % 2]);
        }
    }

    /**
     * Breaks the word into a couple of words
     * @param $word string the word to break
     * @return array an array of word parts
     */
    private function explodeWord($word) {
        $n = mb_strlen($word);
        if ($n < 4) {
            return array($word);
        } else if ($n < 7) {
            $i = $n / 2 + rand(-1, 1);
            return array(mb_substr($word, 0, $i), mb_substr($word, $i));
        } else if ($n < 10) {
            $i1 = rand(1, 3);
            $i2 = $i1 + rand(1, 2);
            return array (
				$this->substring($word, 0, $i1),
				$this->substring($word, $i1, $i2),
				mb_substr($word, $i2)
			);
        } else {
            $i1 = rand(2, 4);
            $i2 = $i1 + rand(1, 3);
			$i3 = $i2 + rand(1, 2);
			return array (
				$this->substring($word, 0, $i1),
				$this->substring($word, $i1, $i2),
				$this->substring($word, $i2, $i3),
				mb_substr($word, $i3)
			);
		}
    }

    private function generateRsid() {
        $digits = array(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'A', 'B', 'C', 'D', 'E', 'F');
        $n = count($digits);
        for ($i = 0, $rsid = ''; $i < 6; $i++) {
            $rsid .= $digits[rand(0, $n - 1)];
        }
        return '00' . $rsid;
    }

    private function getRsids() {
        $rsidsList = $this->xpath->query('//w:rsids/w:rsid');
        $rsids = array();
        foreach ($rsidsList as $rsid) {
            $rsids[] = $this->xpath->query('@w:val', $rsid)->item(0)->nodeValue;
        }
        if (count($rsids) == 0) {
            $rsids = array($this->generateRsid(), $this->generateRsid());
        } else if (count($rsids) == 1) {
            $rsids[] = $this->generateRsid();
        }
        return $rsids;
    }

    private function defineTwoRsids($rsid) {
        $i = array_search($rsid, $this->rsids);
        if ($i == count($this->rsids) - 1) {
            $rsids = array($rsid, $this->rsids[0]);
        } else {
            $rsids = array($rsid, $this->rsids[$i + 1]);
        }
        return $rsids;
    }

    public function __toString() {
        return $this->dom->saveXML();
    }

	public function substring($string, $from, $to) {
		return mb_substr($string, $from, $to - $from);
	}
}